/*
*******************************************************************************
* File:         sample.c
* Description:  Sample Program of E5270A TIS Library
* Revisions:    A.01.00   [ 12/1/2002 ]
* Language:     C
*
* (C) Copyright 2002, Agilent Technologies Company, all rights reserved.
*******************************************************************************
*/
#include <stdio.h>

/*
 *! TIS Bridge Library Header File. This include file must need to use the 
 *! porting library.
 *! When you will port to 4070 TIS, it's should change to 
 *! "/opt/hp4070/include/tis.h"
 */
#include "E5270_TIS.h"  

void Measvth(double Vgstart,double Vgstop,double Vgstep,double Vd,
        double Idt,double Igc,double Idc,double Hold,int pins[],double *Vth)
{
    int    i;
    int    point = (int)(( Vgstop - Vgstart ) / Vgstep + 1.5);
    double measure[1001];
    double source[1001];

#ifdef E5270_TIS_H
    /*
     *! Don't use the SMU port number directly. You should use a variable which
     *! have a SMU port number. It's useful for porting to 4070 system which
     *! is diffrent configuration with E5270. [Recommendation]
     */
    int smuD   = SMU1;
    int smuG   = SMU2;
    int smuS   = SMU3;
    int smuSub = SMU4;
#else
    int smuD   = PORT(0,1);
    int smuG   = PORT(0,2);
    int smuS   = PORT(0,3);
    int smuSub = PORT(0,4);
#endif

    int Drain  = pins[0];
    int Gate   = pins[1];
    int Source = pins[2];
    int Sub    = pins[3];

    /* This function measures Vth using sweep measurement */
    connect_pin(smuD,   Drain);
    connect_pin(smuG,   Gate);
    connect_pin(smuS,   Source);
    connect_pin(smuSub, Sub);

    force_v(Source, 0.0, 20.0, REMAIN); /* Apply zero voltage to Source */
    force_v(Sub, 0.0, 20.0, REMAIN);    /* Apply zero voltage to Substrate */
    force_v(Drain, Vd, 2.0, Idc);    /* Apply Vd to Drain voltage */

    set_iv( Gate, LINEAR_V, 20.0, Vgstart, Vgstop, point, Hold, 0.0, 
            REMAIN, 0.0, COMP_STOP );
    sweep_iv( Drain, I_MEAS, Idc, measure, source, NULL );

    disable_port(Drain);
    disable_port(Gate);
    disable_port(Source);
    disable_port(Sub);
    disconnect_all();

    /* Vth extraction */
    for ( i = 0 ; i < point ; i++ ) {
        if ( measure[i] > Idt ) {
            *Vth = source[i];
            return;
        }
    }
    *Vth = 9999999.99999; /* Target not found */
}

main()
{
    int    ret;
    int    pins[4];
    double vth;

#ifdef E5270_TIS_H
    /*
     * Non compatible function. This function must be called before 
     * use TIS porting library functions.
     */
    ret = open_E5270( "GPIB0::17::INSTR", ERR_DETECT_ON, NULL );
    if ( ret != 0 ) {
        printf( "E5270 device open failed. Exit program.\n" );
        return( -1 );
    }
#endif

    /* Initialize */
    init_system();

    pins[0] = 1;
    pins[1] = 2;
    pins[2] = 3;
    pins[3] = 4;
    Measvth( 0.0, 5.0, 50e-3, 0.0, 1e-3, 1e-3, 20e-3, 0.0, pins, &vth );
    printf( "Measured Vth = %e\n", vth );
    
#ifdef E5270_TIS_H
    close_E5270();  /*
                     *! Instrument will be implicitly closed, when program
                     *! completed. But recommend to call close_E5270() before
                     *! exit program.
                     */
#endif

    return( 0 );
}

